@@ -92,6 +92,7 @@ class OKULAR_EXPORT Annotation
     /// @cond PRIVATE
     friend class AnnotationObjectRect;
     friend class Document;
+    friend class DocumentPrivate;
     friend class Page;
     friend class PagePrivate;
     /// @endcond
@@ -511,16 +512,6 @@ class OKULAR_EXPORT Annotation
                  */
                 QString summary() const;
 
-                /**
-                 * Sets the @p text of the window.
-                 */
-                void setText( const QString &text );
-
-                /**
-                 * Returns the text of the window.
-                 */
-                QString text() const;
-
             private:
                 class Private;
                 Private* const d;
@@ -657,6 +648,20 @@ class OKULAR_EXPORT Annotation
          */
         virtual void store( QDomNode & node, QDomDocument & document ) const;
 
+        /**
+         * Retrieve the QDomNode representing this annotation's properties
+         
+         * @since 0.17 (KDE 4.11)
+         */
+        QDomNode getAnnotationPropertiesDomNode() const;
+
+        /**
+         * Sets annotations internal properties according to the contents of @p node
+         *
+         * @since 0.17 (KDE 4.11)
+         */
+        void setAnnotationProperties( const QDomNode & node );
+
     protected:
         /// @cond PRIVATE
         Annotation( AnnotationPrivate &dd );
@@ -706,7 +711,7 @@ class OKULAR_EXPORT AnnotationProxy
 
         /**
          * Called after an existing @p annotation at a given @p page is modified.
-         * 
+         *
          * Generator can call @p annotation getters to get the new values.
          * @p appearanceChanged tells if a non-visible property was modifed
          *
@@ -800,16 +805,6 @@ class OKULAR_EXPORT TextAnnotation : public Annotation
          */
         int inplaceAlignment() const;
 
-        /**
-         * Sets the inplace @p text of the text annotation.
-         */
-        void setInplaceText( const QString &text );
-
-        /**
-         * Returns the inplace text of the text annotation.
-         */
-        QString inplaceText() const;
-
         /**
          * Sets the inplace callout @p point at @p index.
          *
