@@ -89,6 +89,7 @@
 #include "core/annotations.h"
 #include "core/bookmarkmanager.h"
 #include "core/document.h"
+#include "core/document_p.h"
 #include "core/generator.h"
 #include "core/page.h"
 #include "core/fileprinter.h"
@@ -219,15 +220,12 @@ static QString compressedMimeFor( const QString& mime_to_check )
         return it.value();
 
     KMimeType::Ptr mime = KMimeType::mimeType( mime_to_check );
-    if ( mime )
-    {
-        if ( mime->is( app_gzip ) )
-            return app_gzip;
-        else if ( supportBzip && mime->is( app_bzip ) )
-            return app_bzip;
-        else if ( supportXz && mime->is( app_xz ) )
-            return app_xz;
-    }
+    if ( mime->is( app_gzip ) )
+        return app_gzip;
+    else if ( supportBzip && mime->is( app_bzip ) )
+        return app_bzip;
+    else if ( supportXz && mime->is( app_xz ) )
+        return app_xz;
 
     return QString();
 }
@@ -302,7 +300,7 @@ const QVariantList &args,
 KComponentData componentData )
 : KParts::ReadWritePart(parent),
 m_tempfile( 0 ), m_fileWasRemoved( false ), m_showMenuBarAction( 0 ), m_showFullScreenAction( 0 ), m_actionsSearched( false ),
-m_cliPresentation(false), m_embedMode(detectEmbedMode(parentWidget, parent, args)), m_generatorGuiClient(0), m_keeper( 0 )
+m_cliPresentation(false), m_cliPrint(false), m_embedMode(detectEmbedMode(parentWidget, parent, args)), m_generatorGuiClient(0), m_keeper( 0 )
 {
     // first, we check if a config file name has been specified
     QString configFileName = detectConfigFileName( args );
@@ -431,6 +429,7 @@ m_cliPresentation(false), m_embedMode(detectEmbedMode(parentWidget, parent, args
     m_formsMessage = new PageViewTopMessage( rightContainer );
     rightLayout->addWidget( m_formsMessage );
     m_pageView = new PageView( rightContainer, m_document );
+    m_document->d->m_tiledObserver = m_pageView;
     QMetaObject::invokeMethod( m_pageView, "setFocus", Qt::QueuedConnection );      //usability setting
 //    m_splitter->setFocusProxy(m_pageView);
     connect( m_pageView, SIGNAL(urlDropped(KUrl)), SLOT(openUrlFromDocument(KUrl)));
@@ -503,6 +502,9 @@ m_cliPresentation(false), m_embedMode(detectEmbedMode(parentWidget, parent, args
 
     slotNewConfig();
 
+    // keep us informed when the user changes settings
+    connect( Okular::Settings::self(), SIGNAL(configChanged()), this, SLOT(slotNewConfig()) );
+
     // [SPEECH] check for KTTSD presence and usability
     const KService::Ptr kttsd = KService::serviceByDesktopName("kttsd");
     Okular::Settings::setUseKTTSD( kttsd );
@@ -815,6 +817,11 @@ void Part::setupActions()
     ac->addAction( "presentation_erase_drawings", eraseDrawingAction );
     eraseDrawingAction->setIcon( KIcon( "draw-eraser" ) );
     eraseDrawingAction->setEnabled( false );
+
+    KAction *configureAnnotations = new KAction( i18n( "Configure Annotations..." ), ac );
+    ac->addAction( "options_configure_annotations", configureAnnotations );
+    configureAnnotations->setIcon( KIcon( "configure" ) );
+    connect(configureAnnotations, SIGNAL(triggered()), this, SLOT(slotAnnotationPreferences()));
 }
 
 Part::~Part()
@@ -994,6 +1001,7 @@ void Part::slotJobFinished(KJob *job)
 void Part::loadCancelled(const QString &reason)
 {
     emit setWindowCaption( QString() );
+    resetStartArguments();
 
     // when m_viewportDirty.pageNumber != -1 we come from slotDoFileDirty
     // so we don't want to show an ugly messagebox just because the document is
@@ -1025,15 +1033,12 @@ void Part::setWindowTitleFromDocument()
     emit setWindowCaption( title );
 }
 
-void Part::slotGeneratorPreferences( )
+KConfigDialog * Part::slotGeneratorPreferences( )
 {
-    // an instance the dialog could be already created and could be cached,
-    // in which case you want to display the cached dialog
-    if ( KConfigDialog::showDialog( "generator_prefs" ) )
-        return;
-
-    // we didn't find an instance of this dialog, so lets create it
+    // Create dialog
     KConfigDialog * dialog = new KConfigDialog( m_pageView, "generator_prefs", Okular::Settings::self() );
+    dialog->setAttribute( Qt::WA_DeleteOnClose );
+
     if( m_embedMode == ViewerWidgetMode )
     {
         dialog->setCaption( i18n( "Configure Viewer Backends" ) );
@@ -1045,9 +1050,11 @@ void Part::slotGeneratorPreferences( )
 
     m_document->fillConfigDialog( dialog );
 
-    // keep us informed when the user changes settings
-    connect( dialog, SIGNAL(settingsChanged(QString)), this, SLOT(slotNewGeneratorConfig()) );
+    // Show it
+    dialog->setWindowModality( Qt::ApplicationModal );
     dialog->show();
+
+    return dialog;
 }
 
 
@@ -1324,6 +1331,11 @@ bool Part::openFile()
     m_generatorGuiClient = factory() ? m_document->guiClient() : 0;
     if ( m_generatorGuiClient )
         factory()->addClient( m_generatorGuiClient );
+    if ( m_cliPrint )
+    {
+        m_cliPrint = false;
+        slotPrint();
+    }
     return true;
 }
 
@@ -1366,6 +1378,7 @@ bool Part::openUrl(const KUrl &_url)
     }
     else
     {
+        resetStartArguments();
         KMessageBox::error( widget(), i18n("Could not open %1", url.pathOrUrl() ) );
     }
 
@@ -1559,6 +1572,8 @@ void Part::slotFileDirty( const QString& path )
 
 void Part::slotDoFileDirty()
 {
+    bool tocReloadPrepared = false;
+    
     // do the following the first time the file is reloaded
     if ( m_viewportDirty.pageNumber == -1 )
     {
@@ -1575,6 +1590,10 @@ void Part::slotDoFileDirty()
 
         // store if presentation view was open
         m_wasPresentationOpen = ((PresentationWidget*)m_presentationWidget != 0);
+        
+        // preserves the toc state after reload
+        m_toc->prepareForReload();
+        tocReloadPrepared = true;
 
         // store the page rotation
         m_dirtyPageRotation = m_document->rotation();
@@ -1586,7 +1605,18 @@ void Part::slotDoFileDirty()
 
     // close and (try to) reopen the document
     if ( !closeUrl() )
+    {
+        m_viewportDirty.pageNumber = -1;
+
+        if ( tocReloadPrepared ) 
+        {
+            m_toc->rollbackReload();
+        }
         return;
+    }
+    
+    if ( tocReloadPrepared )
+        m_toc->finishReload();
 
     // inform the user about the operation in progress
     m_pageView->displayMessage( i18n("Reloading the document...") );
@@ -2026,7 +2056,7 @@ void Part::slotSaveFileAs()
         }
     }
 
-    KUrl saveUrl = KFileDialog::getSaveUrl( KUrl("kfiledialog:///okular/" + url().fileName()),
+    KUrl saveUrl = KFileDialog::getSaveUrl( url(),
                                             QString(), widget(), QString(),
                                             KFileDialog::ConfirmOverwrite );
     if ( !saveUrl.isValid() || saveUrl.isEmpty() )
@@ -2140,16 +2170,23 @@ void Part::slotGetNewStuff()
 
 void Part::slotPreferences()
 {
-    // an instance the dialog could be already created and could be cached,
-    // in which case you want to display the cached dialog
-    if ( PreferencesDialog::showDialog( "preferences" ) )
-        return;
-
-    // we didn't find an instance of this dialog, so lets create it
+    // Create dialog
     PreferencesDialog * dialog = new PreferencesDialog( m_pageView, Okular::Settings::self(), m_embedMode );
-    // keep us informed when the user changes settings
-    connect( dialog, SIGNAL(settingsChanged(QString)), this, SLOT(slotNewConfig()) );
+    dialog->setAttribute( Qt::WA_DeleteOnClose );
 
+    // Show it
+    dialog->show();
+}
+
+
+void Part::slotAnnotationPreferences()
+{
+    // Create dialog
+    PreferencesDialog * dialog = new PreferencesDialog( m_pageView, Okular::Settings::self(), m_embedMode );
+    dialog->setAttribute( Qt::WA_DeleteOnClose );
+
+    // Show it
+    dialog->switchToAnnotationsPage();
     dialog->show();
 }
 
@@ -2184,31 +2221,6 @@ void Part::slotNewConfig()
 }
 
 
-void Part::slotNewGeneratorConfig()
-{
-    // Apply settings here. A good policy is to check whether the setting has
-    // changed before applying changes.
-
-    // NOTE: it's not needed to reload the configuration of the Document,
-    // the Document itself will take care of that
-
-    // Main View (pageView)
-    m_pageView->reparseConfig();
-
-    // update TOC settings
-    if ( m_sidebar->isItemEnabled(0) )
-        m_toc->reparseConfig();
-
-    // update ThumbnailList contents
-    if ( Okular::Settings::showLeftPanel() && !m_thumbnailList->isHidden() )
-        m_thumbnailList->updateWidgets();
-
-    // update Reviews settings
-    if ( m_sidebar->isItemEnabled(2) )
-        m_reviewsWidget->reparseConfig();
-}
-
-
 void Part::slotPrintPreview()
 {
     if (m_document->pages() == 0) return;
@@ -2373,6 +2385,10 @@ void Part::reload()
     }
 }
 
+void Part::enableStartWithPrint()
+{
+    m_cliPrint = true;
+}
 
 void Part::slotAboutBackend()
 {
@@ -2753,6 +2769,11 @@ void Part::updateAboutBackendAction()
     }
 }
 
+void Part::resetStartArguments()
+{
+    m_cliPrint = false;
+}
+
 void Part::setReadWrite(bool readwrite)
 {
     m_document->setAnnotationEditingEnabled( readwrite );
