@@ -76,7 +76,6 @@ Q_OBJECT
         void displayMessage( const QString & message, const QString & details = QString(), PageViewMessage::Icon icon=PageViewMessage::Info, int duration=-1 );
 
         // inherited from DocumentObserver
-        uint observerId() const { return PAGEVIEW_ID; }
         void notifySetup( const QVector< Okular::Page * > & pages, int setupFlags );
         void notifyViewportChanged( bool smoothMove );
         void notifyPageChanged( int pageNumber, int changedFlags );
@@ -86,7 +85,6 @@ Q_OBJECT
         void notifyCurrentPageChanged( int previous, int current );
 
         // inherited from View
-        uint viewId() const { return observerId(); }
         bool supportsCapability( ViewCapability capability ) const;
         CapabilityFlags capabilityFlags( ViewCapability capability ) const;
         QVariant capability( ViewCapability capability ) const;
@@ -97,6 +95,7 @@ Q_OBJECT
 
         void reparseConfig();
 
+        KActionCollection *actionCollection() const;
         KAction *toggleFormsAction() const;
 
         int contentAreaWidth() const;
@@ -249,7 +248,7 @@ Q_OBJECT
         void slotPageSizes( int );
         void slotTrimMarginsToggled( bool );
         void slotToggleForms();
-        void slotFormWidgetChanged( FormWidgetIface *w );
+        void slotFormChanged( int pageNumber );
         void slotRefreshPage();
         void slotSpeakDocument();
         void slotSpeakCurrentPage();
